//===================================================================
// pas2000s.c
//-------------------------------------------------------------------
//
// PA-S2000 Sample
//
//-------------------------------------------------------------------
// CH1CALM͂AgKʒȗÕf[^\܂B
//===================================================================
#define	STRICT
#include <windows.h>
#include <tchar.h>
#include <stdio.h>
#include <stdlib.h>

#include "pas2000w.h"


#define WM_PAS2000_BASE			WM_USER + 100
#define WM_PAS2000_DATA_READY	WM_PAS2000_BASE + WM_DATA_READY
#define WM_PAS2000_ERROR		WM_PAS2000_BASE + WM_DATA_THREAD_ERROR

const TCHAR szTitle[] = _T( "Sample" );
const TCHAR szClassName[] = _T( "SampleProgram" );

static void ResultMessage( LPCTSTR lpszbuf );
static void ErrorMessage( WORD devno );
double VoltValue( DWORD addata );

LRESULT CALLBACK WndProc( HWND, UINT, WPARAM, LPARAM );

BOOL	OnCreate( HWND );
BOOL	OnDestroy( void );
BOOL	DataReady( HWND, WPARAM, LPARAM );
BOOL	DataThreadError( WPARAM, LPARAM );

WORD		mwDeviceNo;
PDWORD		mpdwADData;
TSETMODE	mSetMode;

//-------------------------------------------------------------------
// Main
//-------------------------------------------------------------------
int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
					LPSTR lpCmdLine, int nCmdShow )
{
	MSG			msg;
	WNDCLASS	wc;
	HWND		hWnd;

	//---------------------------------------------------------------
	// EChENXo^
	//---------------------------------------------------------------
	if ( !hPrevInstance ) {
		wc.style		 = CS_HREDRAW | CS_VREDRAW;
		wc.lpfnWndProc	 = WndProc;
		wc.cbClsExtra	 = 0;
		wc.cbWndExtra	 = 0;
		wc.hInstance	 = hInstance;
		wc.hIcon		 = NULL;
		wc.hCursor		 = LoadCursor( NULL, IDC_ARROW );
		wc.hbrBackground = (HBRUSH)GetStockObject( WHITE_BRUSH );
		wc.lpszMenuName	 = NULL;
		wc.lpszClassName = szClassName;
		if ( !RegisterClass( &wc ) )
			return FALSE;
	}
	//---------------------------------------------------------------
	// EChE쐬
	//---------------------------------------------------------------
	hWnd = CreateWindow( szClassName,
						 szTitle,
						 WS_OVERLAPPEDWINDOW,
						 CW_USEDEFAULT,
						 CW_USEDEFAULT,
						 150,
						 150,
						 NULL,
						 NULL,
						 hInstance,
						 NULL );
	if ( hWnd == NULL )
		return FALSE;
	//---------------------------------------------------------------
	// EChE\
	//---------------------------------------------------------------
	ShowWindow( hWnd, nCmdShow );
	UpdateWindow( hWnd );
	//---------------------------------------------------------------
	// bZ[W[v
	//---------------------------------------------------------------
	while ( GetMessage( &msg, NULL, 0, 0 ) ) {
		TranslateMessage( &msg );
		DispatchMessage( &msg );
	}
	return ( msg.wParam );
}
//-------------------------------------------------------------------
// EChEbZ[W
//-------------------------------------------------------------------
LRESULT CALLBACK WndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch ( msg ) {
		case WM_SHOWWINDOW:
			OnCreate( hWnd );
			break;
		case WM_PAS2000_DATA_READY:
			DataReady( hWnd, wParam, lParam );
			break;
		case WM_PAS2000_ERROR:
			DataThreadError( wParam, lParam );
			break;
		case WM_DESTROY:
			OnDestroy();
			PostQuitMessage( 0 );
			break;
		default:
			return ( DefWindowProc( hWnd, msg, wParam, lParam ) );
	}
	return 0;
}
//-------------------------------------------------------------------
// 
//-------------------------------------------------------------------
BOOL OnCreate( HWND hWnd )
{
	TCHAR		szbuf[ 1024 ];
	DWORD		dwDrvVer;
	DWORD		dwDllVer;
	DWORD		dwMemSize;
	TDEVINFO	DevInfo;

	//----------------------------------------------------------
	// DLL̎gp錾
	//----------------------------------------------------------
	if ( FALSE == Pas2000wDllOpen() ) {
		ErrorMessage( PAS2000_DEVICE_AUTO );
	}
	//----------------------------------------------------------
	// foCX̎gp錾
	//----------------------------------------------------------
	mwDeviceNo = PAS2000_DEVICE_AUTO;
	if ( FALSE == Pas2000wCreate( &mwDeviceNo, hWnd, WM_PAS2000_BASE ) ) {
		ErrorMessage( PAS2000_DEVICE_AUTO );
	}
	wsprintf( szbuf, _T( "foCX %d ̎gp錾܂B" ), mwDeviceNo );
	ResultMessage( szbuf );
	//----------------------------------------------------------
	// o[W擾
	//----------------------------------------------------------
	if ( FALSE == Pas2000wGetVersion( &dwDrvVer, &dwDllVer ) ) {
		ErrorMessage( PAS2000_DEVICE_AUTO );
	}
	wsprintf( szbuf, _T( "o[W\n")
					 _T( "   Driver Version : %.8lxH\n" )
					 _T( "   DLL Version : %.8lxH" ),
					 dwDrvVer, dwDllVer );
	ResultMessage( szbuf );
	//------------------------------------------------------------
	// foCX擾
	//------------------------------------------------------------
	if ( FALSE == Pas2000wGetDevInfo( mwDeviceNo, &DevInfo ) ) {
		ErrorMessage( mwDeviceNo );
	}
	sprintf( szbuf, _T( "foCX\n" )
					_T( "   Firm Version : %.2f\n" )
					_T( "   FPGA Version : %.2f" ),
					( DevInfo.FirmVer / 100 ) + ( DevInfo.FirmVer % 100 ) * 0.01,
					( DevInfo.FpgaVer / 100 ) + ( DevInfo.FpgaVer % 100 ) * 0.01 );
	ResultMessage( szbuf );
	//------------------------------------------------------------
	// TCY擾
	//------------------------------------------------------------
	if ( FALSE == Pas2000wGetMemSize( mwDeviceNo, &dwMemSize ) ) {
		ErrorMessage( mwDeviceNo );
	}
	wsprintf( szbuf, _T( "TCY\n" ) );
	if ( 0 == dwMemSize ) {
		strcat( szbuf, _T( "   128K|Cg" ) );
	}
	else {
		strcat( szbuf, _T( "   1M|Cg" ) );
	}
	ResultMessage( szbuf );
	//------------------------------------------------------------
	// TvO샂[hݒ
	//------------------------------------------------------------
	// ݒ
	mSetMode.dwVoltDiv[ PAS2000_CH1 ]	   = PAS2000_VOLTDIV_1;		// XP[
	mSetMode.dwVoltDiv[ PAS2000_CH2 ]	   = PAS2000_VOLTDIV_1;
	mSetMode.dwVoltPos[ PAS2000_CH1 ]	   = 100;					// |WV
	mSetMode.dwVoltPos[ PAS2000_CH2 ]	   = 100;
	mSetMode.dwVertCoupling[ PAS2000_CH1 ] = PAS2000_CUP_DC;		// ͌
	mSetMode.dwVertCoupling[ PAS2000_CH2 ] = PAS2000_CUP_DC;
	// ݒ
	mSetMode.dwMemSize					   = PAS2000_MEMSIZE_10K;	// TCY
	mSetMode.dwSamplingTime				   = PAS2000_TIMEDIV_200NS;	// XP[
	mSetMode.dwBandWidth[ PAS2000_CH1 ]    = PAS2000_BW_FULL;		// gш
	mSetMode.dwBandWidth[ PAS2000_CH2 ]    = PAS2000_BW_FULL;
	mSetMode.dwSamplingMode				   = PAS2000_SMP_REAL;		// TvO[h
	mSetMode.dwAcqMode					   = PAS2000_ACQ_SINGLE;	// ANCWV[h
	// gKݒ
	mSetMode.dwTrigMode					   = PAS2000_TMODE_AUTO;	// gK[h
	mSetMode.dwTrigSource				   = PAS2000_TSRC_CH1;		// gK\[X
	mSetMode.dwTrigCoupling				   = PAS2000_TCUP_DC;		// gK͌
	mSetMode.dwTrigSlope				   = PAS2000_TSLP_POS;		// gKX[v
	mSetMode.dwTrigLevel				   = 250;					// gKx
	mSetMode.dwTrigPos					   = 5000;					// gK|WV
	mSetMode.dwTrigDelay				   = 0;						// gKfBC
	mSetMode.dwTrigPulse				   = PAS2000_PULSE_NONE;	// pXgK(pX)
	mSetMode.dwTrigPulseWidth			   = 0;						// pXgK(pX)
	if ( FALSE == Pas2000wSetMode( mwDeviceNo, &mSetMode ) ) {
		ErrorMessage( mwDeviceNo );
	}
	wsprintf( szbuf, _T( "TvO샂[hݒ\n" )
					 _T( " yݒz\n" )
					 _T( "   CH1 XP[ : %d\n" )
					 _T( "   CH2 XP[ : %d\n" )
					 _T( "   CH1 |WV : %d\n" )
					 _T( "   CH2 |WV : %d\n" )
					 _T( "   CH1 ͌ : %d\n" )
					 _T( "   CH2 ͌ : %d\n" )
					 _T( " yݒz\n" )
					 _T( "   TCY : %d\n" )
					 _T( "   TvO^C : %d\n" )
					 _T( "   CH1 gш : %d\n" )
					 _T( "   CH2 gш : %d\n" )
					 _T( "   TvO[h : %d\n" )
					 _T( "   ANCWV[h : %d\n" )
					 _T( " ygKݒz\n" )
					 _T( "   [h : %d\n" )
					 _T( "   \[X : %d\n" )
					 _T( "   ͌ : %d\n" )
					 _T( "   X[v : %d\n" )
					 _T( "   x : %d\n" )
					 _T( "   |WV : %d\n" )
					 _T( "   fBC : %d\n" )
					 _T( "   pX : %d\n" )
					 _T( "   pX : %d" ),
					 mSetMode.dwVoltDiv[ PAS2000_CH1 ],
					 mSetMode.dwVoltDiv[ PAS2000_CH2 ],
					 mSetMode.dwVoltPos[ PAS2000_CH1 ],
					 mSetMode.dwVoltPos[ PAS2000_CH2 ],
					 mSetMode.dwVertCoupling[ PAS2000_CH1 ],
					 mSetMode.dwVertCoupling[ PAS2000_CH2 ],
					 mSetMode.dwMemSize,
					 mSetMode.dwSamplingTime,
					 mSetMode.dwBandWidth[ PAS2000_CH1 ],
					 mSetMode.dwBandWidth[ PAS2000_CH2 ],
					 mSetMode.dwSamplingMode,
					 mSetMode.dwAcqMode,
					 mSetMode.dwTrigMode,
					 mSetMode.dwTrigSource,
					 mSetMode.dwTrigCoupling,
					 mSetMode.dwTrigSlope,
					 mSetMode.dwTrigLevel,
					 mSetMode.dwTrigPos,
					 mSetMode.dwTrigDelay,
					 mSetMode.dwTrigPulse,
					 mSetMode.dwTrigPulseWidth );
	ResultMessage( szbuf );
	//------------------------------------------------------------
	// TvOJn
	//------------------------------------------------------------
	ResultMessage( "TvOJn܂B" );
	if ( FALSE == Pas2000wStartSampling( mwDeviceNo, 0 ) ) {
		ErrorMessage( mwDeviceNo );
	}
	return TRUE;
}
//-------------------------------------------------------------------
// I
//-------------------------------------------------------------------
BOOL OnDestroy( void )
{
	//---------------------------------------------------------------
	// foCX̎gpJ
	//---------------------------------------------------------------
	if ( FALSE == Pas2000wClose( mwDeviceNo ) ) {
		ErrorMessage( mwDeviceNo );
	}
	ResultMessage( "foCXJ܂B" );
	//---------------------------------------------------------------
	// DLL̎gpJ
	//---------------------------------------------------------------
	if ( FALSE == Pas2000wDllClose() ) {
		MessageBox( NULL, _T( "pas2000.dll ̃A[hɎs܂B" ),
					szTitle, MB_OK | MB_ICONSTOP );
		return FALSE;
	}		
	return TRUE;
}
//-------------------------------------------------------------------
// TvOf[^
//-------------------------------------------------------------------
BOOL DataReady( HWND hWnd, WPARAM wParam, LPARAM lParam )
{

	TCHAR	szbuf[ 1024 ];
	TCHAR	sztmp[ 1024 ];
	WORD	wDevNo;
	DWORD	dwStatus;
	DWORD	dwNumData;
	DWORD	dwInitArg;
	DWORD	dwData;
	int		i;

	if ( PAS2000_DEVICE_ID != lParam ) {
		return FALSE;
	}
	wDevNo = (WORD)wParam;
	//----------------------------------------------------------
	// TvO~
	//----------------------------------------------------------
	if ( FALSE == Pas2000wStopSampling( wDevNo ) ) {
		ErrorMessage( wDevNo );
	}
	ResultMessage( "TvO~܂B" );
	//----------------------------------------------------------
	// TvOXe[^X擾
	//----------------------------------------------------------
	if ( FALSE == Pas2000wGetStatus( wDevNo, &dwStatus, &dwNumData, &dwInitArg ) ) {
		ErrorMessage( wDevNo );
	}
	wsprintf( szbuf, _T( "TvOXe[^X擾\n" )
					 _T( "   TvOXe[^X : %xH\n" )
					 _T( "   ǃTvOf[^ : %d" ),
					 dwStatus, dwNumData );
	ResultMessage( szbuf );
	//----------------------------------------------------------
	// TvOf[^擾
	//----------------------------------------------------------
	if ( dwNumData > 0 ) {
		mpdwADData = (PDWORD)malloc( dwNumData * sizeof( DWORD ) );
		memset( mpdwADData, 0, dwNumData * sizeof( DWORD ) );
		if ( FALSE == Pas2000wGetData( wDevNo, dwNumData, mpdwADData ) ) {
			ErrorMessage( wDevNo );
		}
		wsprintf( szbuf, _T( "TvOf[^ %d 擾܂B\n\n" )
						 _T( "CH1 擾f[^\n" ),
						 dwNumData );
		for ( i = 4990; i <= 5010; i++ ) {	// 擾f[^\
			dwData = mpdwADData[ i ] & ADDATA_MASK_BIT;				// CH1
			//dwData = ( mpdwADData[ i ] >> 16 ) & ADDATA_MASK_BIT;	// CH2
			sprintf( sztmp, _T( "   %d : %d ( %.2f )\n" ), 
							i, dwData, VoltValue( dwData ) );
			strcat( szbuf, sztmp );
		}
		wsprintf( sztmp, _T( "\n gKʒu : %d ̑O10\\n" )
						 _T( "@@JbR͓dl(PmV)" ),
						 mSetMode.dwTrigPos );
		strcat( szbuf, sztmp );
		ResultMessage( szbuf );
		free( mpdwADData );
	}
	//----------------------------------------------------------
	// I
	//----------------------------------------------------------
	SendMessage( hWnd, WM_DESTROY, 0, 0 );
	return TRUE;
}
//-------------------------------------------------------------------
// dlZ
//-------------------------------------------------------------------
double VoltValue( DWORD addata )
{
	DWORD	voltdiv;
	double	vfull;

	// Volt/div
	switch ( mSetMode.dwVoltDiv[ PAS2000_CH1 ] ) {
		case PAS2000_VOLTDIV_0:
			voltdiv = 10;
			break;
		case PAS2000_VOLTDIV_1:
			voltdiv = 20;
			break;
		case PAS2000_VOLTDIV_2:
			voltdiv = 50;
			break;
		case PAS2000_VOLTDIV_3:
			voltdiv = 100;
			break;
		case PAS2000_VOLTDIV_4:
			voltdiv = 200;
			break;
		case PAS2000_VOLTDIV_5:
			voltdiv = 500;
			break;
		case PAS2000_VOLTDIV_6:
			voltdiv = 1000;
			break;
		case PAS2000_VOLTDIV_7:
			voltdiv = 2000;
			break;
		default: voltdiv = 0;
	}
	// tXP[d(v[u10:1)
	vfull = voltdiv * PAS2000_PROBE_X10 * 10;
	// dlɊZ
	return ( ( vfull * addata / PAS2000_RES_VERT )
		   - ( vfull * mSetMode.dwVoltPos[ PAS2000_CH1 ] / PAS2000_RES_VOLTPOS ) );
}
//-------------------------------------------------------------------
// TvOf[^擾pXbhG[
//-------------------------------------------------------------------
BOOL DataThreadError( WPARAM wParam, LPARAM lParam )
{
	if ( PAS2000_DEVICE_ID == lParam ) {
		ErrorMessage( (WORD)wParam );
	}
	return TRUE;
}
//-------------------------------------------------------------------
// ʂ\
//-------------------------------------------------------------------
static void ResultMessage( LPCTSTR lpszbuf )
{
	MessageBox( NULL, lpszbuf, szTitle, MB_OK | MB_ICONINFORMATION );
}
//-------------------------------------------------------------------
// G[bZ[W\
//-------------------------------------------------------------------
static void ErrorMessage( WORD devno )
{
	DWORD dwRes = Pas2000wGetLastError( devno );
	TCHAR szbuf[ 50 ];

	if ( PAS2000_ERR_WRAPDLL == dwRes ) {
		wsprintf( szbuf, _T( "DLL Load error" ) );
	}
	else {
		wsprintf( szbuf, _T( "PA-S2000 error\n\n" )
						 _T( "Error code : %.8xH" ),
						 dwRes );
	}
	MessageBox( NULL, szbuf, szTitle, MB_OK | MB_ICONSTOP );
	exit( 0 );
}
