﻿//===================================================================
// pas2000s.cs
//-------------------------------------------------------------------
//
// PA-S2000 Sample
//
//-------------------------------------------------------------------
// CH1にCAL信号を入力し、トリガ位置の前後のデータを表示します。
//===================================================================
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace SamplePas2000
{
    public partial class Pas2000s : Form
    {
        private const int WM_USER = 0x400;
        private const int WM_PAS2000_BASE = WM_USER + 100;
        private const int WM_PAS2000_DATA_READY = WM_PAS2000_BASE + Pas2000.WM_DATA_READY;
        private const int WM_PAS2000_ERROR = WM_PAS2000_BASE + Pas2000.WM_DATA_THREAD_ERROR;

        private Pas2000 Pas2000w;
        private Pas2000.TSETMODE mSetMode;
        private short mwDeviceNo;

        public Pas2000s()
        {
            InitializeComponent();
        }

        //-------------------------------------------------------------------
        // 初期化処理
        //-------------------------------------------------------------------
        private void Pas2000s_Load(object sender, EventArgs e)
        {
            int i;
            int dwDrvVer = 0, dwDllVer = 0, dwMemSize = 0, hwnd;
            short wDeviceNo;
            string szbuf;
            Pas2000.TDEVINFO DevInfo;

            //------------------------------------------------------------
            // 初期化
            //------------------------------------------------------------
            Pas2000w = new Pas2000();
            mSetMode = new Pas2000.TSETMODE();
            mSetMode.dwVoltDiv = new int[Pas2000.NUM_CH];
            mSetMode.dwVoltPos = new int[Pas2000.NUM_CH];
            mSetMode.dwVertCoupling = new int[Pas2000.NUM_CH];
            mSetMode.dwBandWidth = new int[Pas2000.NUM_CH];
            DevInfo = new Pas2000.TDEVINFO();
            DevInfo.VendorID = new byte[Pas2000.VID_LENGTH];
            DevInfo.ProductID = new byte[Pas2000.PID_LENGTH];
            for (i = 0; i < Pas2000.VID_LENGTH; i++)
                DevInfo.VendorID[i] = 0;
            for (i = 0; i < Pas2000.PID_LENGTH; i++)
                DevInfo.ProductID[i] = 0;
            DevInfo.FirmVer = 0;
            DevInfo.FpgaVer = 0;
            //------------------------------------------------------------
            // デバイスの使用を宣言
            //------------------------------------------------------------
            wDeviceNo = Pas2000.DEVICE_AUTO;
            hwnd = this.Handle.ToInt32();			// ウインドウハンドル
            if (false == Pas2000w.Create(ref wDeviceNo, hwnd, WM_PAS2000_BASE))
            {
                ErrorMessage(Pas2000.DEVICE_AUTO);
                return;
            }
            szbuf = String.Format("デバイス {0}  の使用を宣言しました。",
                                    wDeviceNo);
            ResultMessage(szbuf);
            mwDeviceNo = wDeviceNo;
            //------------------------------------------------------------
            // バージョン情報を取得
            //------------------------------------------------------------
            if (false == Pas2000w.GetVersion(ref dwDrvVer, ref dwDllVer))
            {
                ErrorMessage(Pas2000.DEVICE_AUTO);
                return;
            }
            szbuf = String.Concat("≪バージョン情報≫\n",
                                    String.Format("   DRV Version : 0x{0:x8}\n",
                                                    dwDrvVer),
                                    String.Format("   DLL Version : 0x{0:x8}",
                                                    dwDllVer));
            ResultMessage(szbuf);
            //------------------------------------------------------------
            // デバイス情報を取得
            //------------------------------------------------------------
            if (false == Pas2000w.GetDevInfo(mwDeviceNo, ref DevInfo))
            {
                ErrorMessage(mwDeviceNo);
                return;
            }
            szbuf = String.Concat("≪デバイス情報≫\n",
                                    String.Format("   Firm Version : {0:f2}\n",
                                                    DevInfo.FirmVer / 100.0),
                                    String.Format("   FPGA Version : {0:f2} ",
                                                    DevInfo.FpgaVer / 100.0));
            ResultMessage(szbuf);
            //------------------------------------------------------------
            // 実装メモリサイズを取得
            //------------------------------------------------------------
            if (false == Pas2000w.GetMemSize(mwDeviceNo, ref dwMemSize))
            {
                ErrorMessage(mwDeviceNo);
                return;
            }
            szbuf = String.Copy("≪実装メモリサイズ≫\n");
            if (0 == dwMemSize)
            {
                szbuf = String.Concat(szbuf, "   128Kポイント");
            }
            else
            {
                szbuf = String.Concat(szbuf, "   1Mポイント");
            }
            ResultMessage(szbuf);
            //------------------------------------------------------------
            // サンプリング動作モード(垂直軸)を設定
            //------------------------------------------------------------
            mSetMode.dwVoltDiv[Pas2000.CH1] = Pas2000.VOLTDIV_1;	// 垂直軸スケール
            mSetMode.dwVoltDiv[Pas2000.CH2] = Pas2000.VOLTDIV_1;
            mSetMode.dwVoltPos[Pas2000.CH1] = 100;					// 垂直軸ポジション
            mSetMode.dwVoltPos[Pas2000.CH2] = 100;
            mSetMode.dwVertCoupling[Pas2000.CH1] = Pas2000.CUP_DC;	// 垂直軸入力結合
            mSetMode.dwVertCoupling[Pas2000.CH2] = Pas2000.CUP_DC;
            //------------------------------------------------------------
            // サンプリング動作モード(水平軸)を設定
            //------------------------------------------------------------
            mSetMode.dwMemSize = Pas2000.MEMSIZE_10K;				// メモリサイズ
            mSetMode.dwSamplingTime = Pas2000.TIMEDIV_200NS;		// 水平軸スケール
            mSetMode.dwBandWidth[Pas2000.CH1] = Pas2000.BW_FULL;	// 周波数帯域
            mSetMode.dwBandWidth[Pas2000.CH2] = Pas2000.BW_FULL;
            mSetMode.dwSamplingMode = Pas2000.SMP_REAL;				// サンプリングモード
            mSetMode.dwAcqMode = Pas2000.ACQ_SINGLE;				// アクイジションモード
            //------------------------------------------------------------
            // サンプリング動作モード(トリガ)を設定
            //------------------------------------------------------------
            mSetMode.dwTrigMode = Pas2000.TMODE_AUTO;				// トリガモード
            mSetMode.dwTrigSource = Pas2000.TSRC_CH1;				// トリガソース
            mSetMode.dwTrigCoupling = Pas2000.TCUP_DC;				// トリガ入力結合
            mSetMode.dwTrigSlope = Pas2000.TSLP_POS;				// トリガスロープ
            mSetMode.dwTrigLevel = 250;								// トリガレベル
            mSetMode.dwTrigPos = 5000;								// トリガポジション
            mSetMode.dwTrigDelay = 0;								// トリガディレイ
            mSetMode.dwTrigPulse = Pas2000.PULSE_NONE;				// パルストリガ(パルス幅条件)
            mSetMode.dwTrigPulseWidth = 0;							// パルストリガ(パルス幅)
            if (false == Pas2000w.SetMode(mwDeviceNo, ref mSetMode))
            {
                ErrorMessage(mwDeviceNo);
                return;
            }
            szbuf = String.Concat("≪サンプリング動作モード設定≫\n",
                                    " 【垂直軸設定】\n",
                                    String.Format("   CH1 スケール : {0}\n",
                                                        mSetMode.dwVoltDiv[Pas2000.CH1]),
                                    String.Format("   CH2 スケール : {0}\n",
                                                        mSetMode.dwVoltDiv[Pas2000.CH2]),
                                    String.Format("   CH1 ポジション : {0}\n",
                                                        mSetMode.dwVoltPos[Pas2000.CH1]),
                                    String.Format("   CH2 ポジション : {0}\n",
                                                        mSetMode.dwVoltPos[Pas2000.CH2]),
                                    String.Format("   CH1 入力結合 : {0}\n",
                                                        mSetMode.dwVertCoupling[Pas2000.CH1]),
                                    String.Format("   CH2 入力結合 : {0}\n",
                                                        mSetMode.dwVertCoupling[Pas2000.CH2]),
                                    " 【水平軸設定】\n",
                                    String.Format("   メモリサイズ : {0}\n",
                                                        mSetMode.dwMemSize),
                                    String.Format("   サンプリングタイム : {0}\n",
                                                        mSetMode.dwSamplingTime),
                                    String.Format("   CH1 周波数帯域 : {0}\n",
                                                        mSetMode.dwBandWidth[Pas2000.CH1]),
                                    String.Format("   CH2 周波数帯域 : {0}\n",
                                                        mSetMode.dwBandWidth[Pas2000.CH2]),
                                    String.Format("   サンプリングモード : {0}\n",
                                                        mSetMode.dwSamplingMode),
                                    String.Format("   アクイジションモード : {0}\n",
                                                        mSetMode.dwAcqMode),
                                    " 【トリガ設定】\n",
                                    String.Format("   モード : {0}\n",
                                                        mSetMode.dwTrigMode),
                                    String.Format("   ソース : {0}\n",
                                                        mSetMode.dwTrigSource),
                                    String.Format("   入力結合 : {0}\n",
                                                        mSetMode.dwTrigCoupling),
                                    String.Format("   スロープ : {0}\n",
                                                        mSetMode.dwTrigSlope),
                                    String.Format("   レベル : {0}\n",
                                                        mSetMode.dwTrigLevel),
                                    String.Format("   ポジション : {0}\n",
                                                        mSetMode.dwTrigPos),
                                    String.Format("   ディレイ : {0}\n",
                                                        mSetMode.dwTrigDelay),
                                    String.Format("   パルス幅条件 : {0}\n",
                                                        mSetMode.dwTrigPulse),
                                    String.Format("   パルス幅 : {0}\n",
                                                        mSetMode.dwTrigPulseWidth));
            ResultMessage(szbuf);
            //------------------------------------------------------------
            // サンプリングを開始
            //------------------------------------------------------------
            ResultMessage("サンプリングを開始します。");
            if (false == Pas2000w.StartSampling(mwDeviceNo, 0))
            {
                ErrorMessage(mwDeviceNo);
                return;
            }
        }

        //-------------------------------------------------------------------
        // 終了処理
        //-------------------------------------------------------------------
        private void Pas2000s_FormClosed(object sender, FormClosedEventArgs e)
        {
            // デバイスの使用を開放
            if (false == Pas2000w.Close(mwDeviceNo))
                ErrorMessage(mwDeviceNo);
            ResultMessage("デバイスを開放しました。");
        }

        //-------------------------------------------------------------------
        // メッセージの処理
        //-------------------------------------------------------------------
        protected override void WndProc(ref Message m)
        {
            switch (m.Msg)
            {
                case WM_PAS2000_DATA_READY:
                    DataReady(m.WParam, m.LParam);
                    break;
                case WM_PAS2000_ERROR:
                    DataThreadError(m.WParam, m.LParam);
                    break;
            }
            base.WndProc(ref m);
        }

        //-------------------------------------------------------------------
        // 描画データ準備完了処理
        //-------------------------------------------------------------------
        private void DataReady(IntPtr wParam, IntPtr lParam)
        {
            string szbuf;
            string sztmp;
            int dwData, dwStatus = 0, dwNumData = 0, dwInitArg = 0;
            double dblData;
            short wDevNo;
            int i;

            if (Pas2000.DEVICE_ID != lParam.ToInt32())
                return;
            wDevNo = (short)wParam.ToInt32();
            //------------------------------------------------------
            // サンプリングを停止
            //------------------------------------------------------
            if (false == Pas2000w.StopSampling(wDevNo))
            {
                ErrorMessage(wDevNo);
            }
            ResultMessage("サンプリングを停止しました。");
            //------------------------------------------------------
            // サンプリングステータスを取得
            //------------------------------------------------------
            if (false == Pas2000w.GetStatus(wDevNo, ref dwStatus, ref dwNumData, ref dwInitArg))
            {
                ErrorMessage(wDevNo);
                return;
            }
            szbuf = String.Concat("≪サンプリングステータス取得≫\n",
                                    String.Format("   サンプリングステータス : 0x{0:x}\n", dwStatus),
                                    String.Format("   未読サンプリングデータ数 : {0}", dwNumData));
            ResultMessage(szbuf);
            //------------------------------------------------------
            // サンプリングデータを取得
            //------------------------------------------------------
            if (dwNumData == 0)
                return;
            int[] dwAdData = new int[dwNumData];
            if (false == Pas2000w.GetData(wDevNo, dwNumData, dwAdData))
            {
                ErrorMessage(wDevNo);
                return;
            }
            szbuf = String.Concat("サンプリングデータを ", dwNumData.ToString(), " 個取得しました。\n\n",
                                    "≪CH1 取得データ≫\n");
            for (i = 4990; i <= 5010; i++)					// 取得データを表示
            {
                dwData = dwAdData[i] & Pas2000.ADDATA_MASK_BIT;			// CH1
                //dwData = (dwAdData[ i ] >> 16) & Pas2000.ADDATA_MASK_BIT;	// CH2
                dblData = VoltValue(dwData);
                sztmp = String.Format("   {0} : {1} ( {2:f2} )\n", i, dwData, dblData);
                szbuf = String.Concat(szbuf, sztmp);
            }
            szbuf = String.Concat(szbuf,
                                    "\n ※トリガ位置 : ",
                                    mSetMode.dwTrigPos.ToString(),
                                    " の前後10個を表示",
                                    "\n　　カッコ内は電圧値(単位mV)");
            ResultMessage(szbuf);
            //----------------------------------------------------------
            // 終了処理
            //----------------------------------------------------------
            Close();
        }

        //-------------------------------------------------------------------
        // 電圧値換算
        //-------------------------------------------------------------------
        private double VoltValue(int addata)
        {
            int voltdiv;
            double vfull;

            // Volt/div
            switch (mSetMode.dwVoltDiv[Pas2000.CH1])
            {
                case Pas2000.VOLTDIV_0:
                    voltdiv = 10;
                    break;
                case Pas2000.VOLTDIV_1:
                    voltdiv = 20;
                    break;
                case Pas2000.VOLTDIV_2:
                    voltdiv = 50;
                    break;
                case Pas2000.VOLTDIV_3:
                    voltdiv = 100;
                    break;
                case Pas2000.VOLTDIV_4:
                    voltdiv = 200;
                    break;
                case Pas2000.VOLTDIV_5:
                    voltdiv = 500;
                    break;
                case Pas2000.VOLTDIV_6:
                    voltdiv = 1000;
                    break;
                case Pas2000.VOLTDIV_7:
                    voltdiv = 2000;
                    break;
                default:
                    voltdiv = 0;
                    break;
            }
            // フルスケール電圧(プローブ減衰率は10:1)
            vfull = voltdiv * Pas2000.PROBE_X10 * 10;
            // 電圧値に換算
            return ((vfull * addata / Pas2000.RES_VERT)
                        - (vfull * mSetMode.dwVoltPos[Pas2000.CH1] / Pas2000.RES_VOLTPOS));
        }

        //-------------------------------------------------------------------
        // 描画データ取得用スレッド内エラー処理
        //-------------------------------------------------------------------
        private void DataThreadError(IntPtr wParam, IntPtr lParam)
        {
            if (Pas2000.DEVICE_ID == lParam.ToInt32())
                ErrorMessage((short)wParam.ToInt32());
        }

        //-------------------------------------------------------------------
        // 処理結果を表示
        //-------------------------------------------------------------------
        private void ResultMessage(string pstrbuf)
        {
            MessageBox.Show(pstrbuf, this.Text,
                                MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        //-------------------------------------------------------------------
        // エラーメッセージを表示
        //-------------------------------------------------------------------
        private void ErrorMessage(short devno)
        {
            int dwRes = Pas2000w.GetLastError(devno);
            string strbuf;

            strbuf = String.Concat("PA-S2000 error\n\n",
                                    String.Format("Error code: 0x{0:x8}", dwRes));
            MessageBox.Show(strbuf, this.Text,
                                MessageBoxButtons.OK, MessageBoxIcon.Stop);
        }
    }
}
