﻿'===================================================================
' pas2000s.vb
'-------------------------------------------------------------------
'
' PA-S2000 Sample
'
'-------------------------------------------------------------------
' CH1にCAL信号を入力し、トリガ位置の前後のデータを表示します。
'===================================================================

Option Strict Off
Option Explicit On

Public Class Pas2000s

    Const WM_USER As Integer = &H400
    Const WM_PAS2000_BASE As Integer = WM_USER + 100
    Const WM_PAS2000_DATA_READY As Integer = WM_PAS2000_BASE + Pas2000.WM_DATA_READY
    Const WM_PAS2000_ERROR As Integer = WM_PAS2000_BASE + Pas2000.WM_DATA_THREAD_ERROR

    Private Shared mwDeviceNo As Short
    Private Shared Pas2000w As Pas2000
    Private Shared mSetMode As Pas2000.TSETMODE

    '-------------------------------------------------------------------
    ' 初期化処理
    '-------------------------------------------------------------------
    Private Sub Pas2000s_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Dim i As Integer
        Dim dwDrvVer As Integer, dwDllVer As Integer
        Dim dwMemSize As Integer, hwnd As Integer
        Dim wDeviceNo As Short
        Dim szbuf As String
        Dim DevInfo As Pas2000.TDEVINFO

        '------------------------------------------------------------
        ' 初期化
        '------------------------------------------------------------
        Pas2000w = New Pas2000()
        mSetMode = New Pas2000.TSETMODE()
        mSetMode.dwVoltDiv = New Integer(Pas2000.NUM_CH - 1) {}
        mSetMode.dwVoltPos = New Integer(Pas2000.NUM_CH - 1) {}
        mSetMode.dwVertCoupling = New Integer(Pas2000.NUM_CH - 1) {}
        mSetMode.dwBandWidth = New Integer(Pas2000.NUM_CH - 1) {}
        DevInfo = New Pas2000.TDEVINFO
        DevInfo.VendorID = New Byte(Pas2000.VID_LENGTH - 1) {}
        DevInfo.ProductID = New Byte(Pas2000.PID_LENGTH - 1) {}
        For i = 0 To Pas2000.VID_LENGTH - 1
            DevInfo.VendorID(i) = 0
        Next
        For i = 0 To Pas2000.PID_LENGTH - 1
            DevInfo.ProductID(i) = 0
        Next
        DevInfo.FirmVer = 0
        DevInfo.FpgaVer = 0
        '------------------------------------------------------------
        ' デバイスの使用を宣言
        '------------------------------------------------------------
        wDeviceNo = Pas2000.DEVICE_AUTO
        hwnd = Me.Handle.ToInt32            ' ウインドウハンドル
        If False = Pas2000w.Create(wDeviceNo, hwnd, WM_PAS2000_BASE) Then
            ErrorMessage(Pas2000.DEVICE_AUTO)
            Exit Sub
        End If
        szbuf = String.Format("デバイス {0} の使用を宣言しました。", wDeviceNo)
        ResultMessage(szbuf)
        mwDeviceNo = wDeviceNo
        '------------------------------------------------------------
        ' バージョン情報を取得
        '------------------------------------------------------------
        If False = Pas2000w.GetVersion(dwDrvVer, dwDllVer) Then
            ErrorMessage(Pas2000.DEVICE_AUTO)
            Exit Sub
        End If
        szbuf = String.Concat("≪バージョン情報≫", vbCrLf, _
                                String.Format("   DRV Version : 0x{0:x8}", dwDrvVer), vbCrLf, _
                                String.Format("   DLL Version : 0x{0:x8}", dwDllVer))
        ResultMessage(szbuf)
        '------------------------------------------------------------
        ' デバイス情報を取得
        '------------------------------------------------------------
        If False = Pas2000w.GetDevInfo(mwDeviceNo, DevInfo) Then
            ErrorMessage(mwDeviceNo)
            Exit Sub
        End If
        szbuf = String.Concat("≪デバイス情報≫", vbCrLf, _
                                String.Format("   Firm Version : {0:f2}", _
                                                DevInfo.FirmVer / 100.0), vbCrLf, _
                                String.Format("   FPGA Version : {0:f2}", _
                                                DevInfo.FpgaVer / 100.0))
        ResultMessage(szbuf)
        '------------------------------------------------------------
        ' 実装メモリサイズを取得
        '------------------------------------------------------------
        If False = Pas2000w.GetMemSize(mwDeviceNo, dwMemSize) Then
            ErrorMessage(mwDeviceNo)
            Exit Sub
        End If
        szbuf = String.Copy("≪実装メモリサイズ≫" & vbCrLf)
        If 0 = dwMemSize Then
            szbuf = String.Concat(szbuf, "   128Kポイント")
        Else
            szbuf = String.Concat(szbuf, "   1Mポイント")
        End If
        ResultMessage(szbuf)
        '------------------------------------------------------------
        ' サンプリング動作モード(垂直軸)を設定
        '------------------------------------------------------------
        mSetMode.dwVoltDiv(Pas2000.CH1) = Pas2000.VOLTDIV_1     ' 垂直軸スケール
        mSetMode.dwVoltDiv(Pas2000.CH2) = Pas2000.VOLTDIV_1
        mSetMode.dwVoltPos(Pas2000.CH1) = 100                   ' 垂直軸ポジション
        mSetMode.dwVoltPos(Pas2000.CH2) = 100
        mSetMode.dwVertCoupling(Pas2000.CH1) = Pas2000.CUP_DC   ' 垂直軸入力結合
        mSetMode.dwVertCoupling(Pas2000.CH2) = Pas2000.CUP_DC
        '------------------------------------------------------------
        ' サンプリング動作モード(水平軸)を設定
        '------------------------------------------------------------
        mSetMode.dwMemSize = Pas2000.MEMSIZE_10K                ' メモリサイズ
        mSetMode.dwSamplingTime = Pas2000.TIMEDIV_200NS         ' 水平軸スケール
        mSetMode.dwBandWidth(Pas2000.CH1) = Pas2000.BW_FULL     ' 周波数帯域
        mSetMode.dwBandWidth(Pas2000.CH2) = Pas2000.BW_FULL
        mSetMode.dwSamplingMode = Pas2000.SMP_REAL              ' サンプリングモード
        mSetMode.dwAcqMode = Pas2000.ACQ_SINGLE                 ' アクイジションモード
        '------------------------------------------------------------
        ' サンプリング動作モード(トリガ)を設定
        '------------------------------------------------------------
        mSetMode.dwTrigMode = Pas2000.TMODE_AUTO                ' トリガモード
        mSetMode.dwTrigSource = Pas2000.TSRC_CH1                ' トリガソース
        mSetMode.dwTrigCoupling = Pas2000.TCUP_DC               ' トリガ入力結合
        mSetMode.dwTrigSlope = Pas2000.TSLP_POS                 ' トリガスロープ
        mSetMode.dwTrigLevel = 250                              ' トリガレベル
        mSetMode.dwTrigPos = 5000                               ' トリガポジション
        mSetMode.dwTrigDelay = 0                                ' トリガディレイ
        mSetMode.dwTrigPulse = Pas2000.PULSE_NONE               ' パルストリガ(パルス幅条件)
        mSetMode.dwTrigPulseWidth = 0                           ' パルストリガ(パルス幅)
        If False = Pas2000w.SetMode(mwDeviceNo, mSetMode) Then
            ErrorMessage(mwDeviceNo)
            Exit Sub
        End If
        szbuf = String.Concat("≪サンプリング動作モード設定≫", vbCrLf, _
                                " 【垂直軸設定】", vbCrLf, _
                                String.Format("   CH1 スケール : {0}", _
                                                mSetMode.dwVoltDiv(Pas2000.CH1)), vbCrLf, _
                                String.Format("   CH2 スケール : {0}", _
                                                mSetMode.dwVoltDiv(Pas2000.CH2)), vbCrLf, _
                                String.Format("   CH1 ポジション : {0}", _
                                                mSetMode.dwVoltPos(Pas2000.CH1)), vbCrLf, _
                                String.Format("   CH2 ポジション : {0}", _
                                                mSetMode.dwVoltPos(Pas2000.CH2)), vbCrLf, _
                                String.Format("   CH1 入力結合 : {0}", _
                                                    mSetMode.dwVertCoupling(Pas2000.CH1)), vbCrLf, _
                                String.Format("   CH2 入力結合 : {0}", _
                                                    mSetMode.dwVertCoupling(Pas2000.CH2)), vbCrLf, _
                                " 【水平軸設定】", vbCrLf, _
                                String.Format("   メモリサイズ : {0}", _
                                                    mSetMode.dwMemSize), vbCrLf, _
                                String.Format("   サンプリングタイム : {0}", _
                                                    mSetMode.dwSamplingTime), vbCrLf, _
                                String.Format("   CH1 周波数帯域 : {0}", _
                                                    mSetMode.dwBandWidth(Pas2000.CH1)), vbCrLf, _
                                String.Format("   CH2 周波数帯域 : {0}", _
                                                    mSetMode.dwBandWidth(Pas2000.CH2)), vbCrLf, _
                                String.Format("   サンプリングモード : {0}", _
                                                    mSetMode.dwSamplingMode), vbCrLf, _
                                String.Format("   アクイジションモード : {0}", _
                                                    mSetMode.dwAcqMode), vbCrLf, _
                                " 【トリガ設定】", vbCrLf, _
                                String.Format("   モード : {0}", _
                                                    mSetMode.dwTrigMode), vbCrLf, _
                                String.Format("   ソース : {0}", _
                                                    mSetMode.dwTrigSource), vbCrLf, _
                                String.Format("   入力結合 : {0}", _
                                                    mSetMode.dwTrigCoupling), vbCrLf, _
                                String.Format("   スロープ : {0}", _
                                                    mSetMode.dwTrigSlope), vbCrLf, _
                                String.Format("   レベル : {0}", _
                                                    mSetMode.dwTrigLevel), vbCrLf, _
                                String.Format("   ポジション : {0}", _
                                                    mSetMode.dwTrigPos), vbCrLf, _
                                String.Format("   ディレイ : {0}", _
                                                    mSetMode.dwTrigDelay), vbCrLf, _
                                String.Format("   パルス幅条件 : {0}", _
                                                    mSetMode.dwTrigPulse), vbCrLf, _
                                String.Format("   パルス幅 : {0}", _
                                                    mSetMode.dwTrigPulseWidth))
        ResultMessage(szbuf)
        '------------------------------------------------------------
        ' サンプリングを開始
        '------------------------------------------------------------
        ResultMessage("サンプリングを開始します。")
        If False = Pas2000w.StartSampling(mwDeviceNo, 0) Then
            ErrorMessage(mwDeviceNo)
            Exit Sub
        End If

    End Sub

    '-------------------------------------------------------------------
    ' 終了処理
    '-------------------------------------------------------------------
    Private Sub Pas2000s_FormClosed(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles MyBase.FormClosed
        ' デバイスの使用を開放
        If False = Pas2000w.Close(mwDeviceNo) Then
            ErrorMessage(mwDeviceNo)
        End If
        ResultMessage("デバイスを開放しました。")

    End Sub

    '-------------------------------------------------------------------
    ' メッセージの処理
    '-------------------------------------------------------------------
    Protected Overrides Sub WndProc(ByRef m As System.Windows.Forms.Message)

        Select Case m.Msg
            Case WM_PAS2000_DATA_READY
                Call DataReady(m.WParam, m.LParam)
            Case WM_PAS2000_ERROR
                Call DataThreadError(m.WParam, m.LParam)
        End Select
        MyBase.WndProc(m)

    End Sub

    '-------------------------------------------------------------------
    ' 描画データ準備完了処理
    '-------------------------------------------------------------------
    Private Sub DataReady(ByVal wParam As IntPtr, ByVal lParam As IntPtr)
        Dim szbuf As String, sztmp As String
        Dim dwData As Integer, dwStatus As Integer, dwNumData As Integer, dwInitArg As Integer
        Dim dblData As Double
        Dim wDevNo As Short
        Dim i As Integer
        Dim dwAdData() As Integer

        If lParam.ToInt32() <> Pas2000.DEVICE_ID Then
            Exit Sub
        End If
        wDevNo = CShort(wParam.ToInt32())
        '------------------------------------------------------
        ' サンプリングを停止
        '------------------------------------------------------
        If False = Pas2000w.StopSampling(wDevNo) Then
            ErrorMessage(wDevNo)
            Exit Sub
        End If
        ResultMessage("サンプリングを停止しました。")
        '------------------------------------------------------
        ' サンプリングステータスを取得
        '------------------------------------------------------
        If False = Pas2000w.GetStatus(wDevNo, dwStatus, dwNumData, dwInitArg) Then
            ErrorMessage(wDevNo)
            Exit Sub
        End If
        szbuf = String.Concat("≪サンプリングステータス取得≫", vbCrLf, _
                                String.Format("   サンプリングステータス : &H{0:x}", _
                                                dwStatus), vbCrLf, _
                                String.Format("   未読サンプリングデータ数 : {0}", _
                                                dwNumData))
        ResultMessage(szbuf)
        '------------------------------------------------------
        ' サンプリングデータを取得
        '------------------------------------------------------
        If dwNumData = 0 Then
            Exit Sub
        End If
        ReDim dwAdData(dwNumData)
        If False = Pas2000w.GetData(wDevNo, dwNumData, dwAdData) Then
            ErrorMessage(wDevNo)
            Exit Sub
        End If
        szbuf = String.Concat("サンプリングデータを ", dwNumData.ToString(), _
                                " 個取得しました。", vbCrLf, vbCrLf, _
                                "≪CH1 取得データ≫", vbCrLf)
        For i = 4990 To 5010  ' 取得データを表示
            dwData = dwAdData(i) And Pas2000.ADDATA_MASK_BIT            ' CH1
            'dwData = (dwAdData(i) / &H1000) And Pas2000.ADDATA_MASK_BIT ' CH2
            dblData = VoltValue(dwData)
            sztmp = String.Format("   {0} : {1} ({2:f2})", _
                                    i, dwData, dblData)
            szbuf = String.Concat(szbuf, sztmp, vbCrLf)
        Next
        szbuf = String.Concat(szbuf, vbCrLf, _
                                " ※トリガ位置 : ", mSetMode.dwTrigPos.ToString(), _
                                " の前後10個を表示", vbCrLf, _
                                "　　カッコ内は電圧値(単位mV)")
        ResultMessage(szbuf)
        '----------------------------------------------------------
        ' 終了処理
        '----------------------------------------------------------
        Me.Close()

    End Sub

    '-------------------------------------------------------------------
    ' 電圧値換算
    '-------------------------------------------------------------------
    Private Function VoltValue(ByVal addata As Integer) As Double
        Dim voltdiv As Integer
        Dim vfull As Double

        ' Volt/div
        Select Case mSetMode.dwVoltDiv(Pas2000.CH1)
            Case Pas2000.VOLTDIV_0
                voltdiv = 10
            Case Pas2000.VOLTDIV_1
                voltdiv = 20
            Case Pas2000.VOLTDIV_2
                voltdiv = 50
            Case Pas2000.VOLTDIV_3
                voltdiv = 100
            Case Pas2000.VOLTDIV_4
                voltdiv = 200
            Case Pas2000.VOLTDIV_5
                voltdiv = 500
            Case Pas2000.VOLTDIV_6
                voltdiv = 1000
            Case Pas2000.VOLTDIV_7
                voltdiv = 2000
            Case Else
                voltdiv = 0
        End Select
        ' フルスケール電圧(プローブ減衰率は10:1)
        vfull = voltdiv * Pas2000.PROBE_X10 * 10
        ' 電圧値に換算
        Return ((vfull * addata / Pas2000.RES_VERT) _
              - (vfull * mSetMode.dwVoltPos(Pas2000.CH1) / Pas2000.RES_VOLTPOS))

    End Function

    '-------------------------------------------------------------------
    ' 描画データ取得用スレッド内エラー処理
    '-------------------------------------------------------------------
    Private Sub DataThreadError(ByVal wParam As IntPtr, ByVal lParam As IntPtr)

        If lParam.ToInt32() = Pas2000.DEVICE_ID Then
            ErrorMessage(CShort(wParam.ToInt32()))
        End If

    End Sub

    '-------------------------------------------------------------------
    ' 処理結果を表示
    '-------------------------------------------------------------------
    Private Sub ResultMessage(ByRef lpszbuf As String)

        MsgBox(lpszbuf, MsgBoxStyle.OKOnly + MsgBoxStyle.Information, Me.Text)

    End Sub

    '-------------------------------------------------------------------
    ' エラーメッセージを表示
    '-------------------------------------------------------------------
    Private Sub ErrorMessage(ByRef devno As Short)
        Dim dwRes As Integer
        Dim szbuf As String

        dwRes = Pas2000w.GetLastError(devno)
        szbuf = String.Concat("PA-S2000 error", vbCrLf, vbCrLf, _
                                String.Format("Error code: &H{0:x8}", dwRes))
        MsgBox(szbuf, MsgBoxStyle.OKOnly + MsgBoxStyle.Critical, Me.Text)

    End Sub

End Class
